/*********************************************************************
 * TTCE 0.1.0: Taxes, Tips and Currencies in Euro
 * (C) Jrme Euzenat, 1998, 2000
 *
 * Parts:
 * (C) Alan Harder, 1999
 * From TipMe v1.23 - 3/29/99
 * Alan.Harder@Sun.COM     http://www.moshpit.org/pilot/
 * Freeware - feel free to distribute and/or modify this code.. just
 *            leave my name on it.
 * 
 *  Units V1.1 - A general units converter
 * Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)
 * 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************/

/************************
 * IMPROVEMENT
 * 
 * It would be nice to simptoly put the field numbers as field handlers and let the 
 * system play ???
 */

/************************
 * CODING ISSUES
 * We want to optimize size, not speed
 *
 * From the given examples, we must determines if:
 * - It is better to compute things twice or to store them in variables (FP seem to think that variables are better)
 * - It is better to have multiple handlers or only one big handler (AH the latter)
 * - What is the price for a form
 * - Is it possible to share menus and so on...
 */

#include "ttce.h"
#include <GuiTools.h>
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#define CREATOR		'TTCE'
#define PREF_VERSION	1

#define TRUE 1
#define FALSE 0
#define ERROR -1
#define MAIN 0

// This takes into account default values, very good!!
Char deciSep[] = ".", deciNoDime[] = ".0";
Char deciChr = '.';

Long total, amount, tip, tax, tax2, tipbase, ctotal;
FieldPtr amountFld, taxFld, taxAmtFld, tax2Fld, tax2AmtFld,
         tipFld, tipAmtFld, totalFld, currFld, currAmtFld, gbMainTestPopTriggerSelected=NULL;

// App Preferences Type
//
typedef struct
{
    Int taxRate, tipRate, tax2Rate, currRate;
    Boolean useTax;
}
TipPrefsType;

TipPrefsType *prefs;

//-- taxRate, tipRate, tax2Rate, currRate;

#define COMPA() if(amount>0)PutAmountInField(taxFld, tax*100/amount);
#define COMPB() if((amount+tax)>0)PutAmountInField(tax2Fld,tax2*100/(amount+tax));
#define COMPC() if(amount>0)PutAmountInField(tipFld,tip*100/amount);
#define COMPD() if(total>0)PutAmountInField(currFld,ctotal/total);

#define RETROCOMPALPHA() amount=(amount*total)/(amount+tip+tax+tax2);
#define RETROCOMPEPSILON() if((prefs->currRate)>0)total=ctotal/prefs->currRate;

//-- Used to display the symbol of the pivot currency
static CharPtr currSymbols[5] = {"", "$", "", "", ""};
static int currIndex = 0;

// Methods to get and set Amounts in text fields.  An amount is the
// number of CENTS, and will be displayed with the decimal point placed
// appropriately..
//
void PutAmountInField(FieldPtr fieldPtr, Long amount)
{
    CharPtr str;
    Long amt;

    str = (CharPtr)MemPtrNew(16);
    if (amount < 0)
    {
	amt = -amount;
	StrCopy(str, "(");
    }
    else
    {
	amt = amount;
	StrCopy(str, "");
    }
    StrIToA(str + StrLen(str), amt/100);
    StrCat(str, (amt%100 < 10) ? deciNoDime : deciSep);
    StrIToA(str + StrLen(str), amt%100);
    if (amount < 0) StrCat(str, ")");
    SetText(fieldPtr, str);
    MemPtrFree(str);
}

Long GetAmountFromField(FieldPtr fieldPtr)
{
    Long val;
    Int cents;
    CharPtr str, str2;
    str = GetText(fieldPtr);
    if (str == NULL) return ERROR;
    if (StrLen(str) == 0) { MemPtrFree(str); return ERROR; }
    val = StrAToI(str) * 100;
    str2 = StrChr(str, deciChr);
    if (str2 != NULL && str2[1] >= '0' && str2[1] <= '9')
    {
	cents = (str2[1] - '0') * 10;
	if (str2[2] >= '0' && str2[2] <= '9')
	    cents += str2[2] - '0';
	val += cents;
    }
    MemPtrFree(str);
    return val;
}


Boolean TaxFormEventHandler(EventType *pEvent)
{
    // a large part of the handler will be shared by 3 editors
    // it can be put in a default handler
  switch (pEvent->eType)
  {
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    // test if new or edit...
    SetFocus(nameEditField); //if new else, put on value
    return TRUE;

  case ctlSelectEvent:
    switch (pEvent->data.ctlSelect.controlID)
    {
    case cancelEditButton:
      FrmGotoForm(editListForm);
      return TRUE;
    case resetEditButton:
      return TRUE;
    case saveEditButton:
      FrmGotoForm(mainForm);
      return TRUE;
      // There are also fields, checkBox and popUpLists...
      //-- For the currency we must have
    case pivotCurrSymbolField:
      currIndex = (currIndex+1) % 5;
      CtlSetLabel( (ControlPtr)GetObjectPtrFromForm(FrmGetActiveForm(), pivotCurrSymbolField), currSymbols[currIndex]);
      return TRUE;
    }
  }

  return FALSE;
}

void computeValues() {
	Long taxRate, tax2Rate, tipRate, currRate;
    // This button shall be obsolete soon but the rounding that have been applied
    //  could be very useful for our comptations
    amount = GetAmountFromField(amountFld);
    if ( (taxRate = GetAmountFromField(taxFld)) > 0) tax = (amount*taxRate+5000)/10000;	// standard rounding
    else  tax = 0;
    PutAmountInField(taxAmtFld, tax);
    if ( (tax2Rate = GetAmountFromField(tax2Fld)) > 0) tax2 = (amount*tax2Rate+5000)/10000;	// standard rounding
    else  tax2 = 0;
    PutAmountInField(tax2AmtFld, tax2);
    if ( (tipRate = GetAmountFromField(tipFld)) > 0) tip = (amount*tipRate+9999)/10000;	// always round up
    else  tip = 0;
    PutAmountInField(tipAmtFld, tip);
    total = amount + tax + tax2 + tip;
    PutAmountInField(totalFld, total);
    if ( (currRate = GetAmountFromField(currFld)) > 0) ctotal = (total * currRate+500000)/10000; // Beware of overflow
    PutAmountInField(currAmtFld, ctotal);
    FrmDrawForm(FrmGetActiveForm());
}


Boolean MenuEventHandler(EventPtr event)
{
    switch ( event->data.menu.itemID ) {
      case menuItemAbout:
	FrmPopupForm(aboutForm) ;
        return TRUE;
      case menuItemEdit:
	FrmPopupForm(editListForm) ;
        return TRUE;
      case menuItemCut:
      case menuItemCopy:
      case menuItemPaste:
	{
	    FormPtr form ;
	    Word tmp_index ;
	    FieldPtr tmp_fld ;
	    
	    /* JE//: from FP, certainly size optimization that I do not know */
	    form = FrmGetActiveForm () ;
	    tmp_index = FrmGetFocus (form) ;
	    if (tmp_index != -1) {
		/* On croise les doigts pour qu'il n'y ait que les */
		/* fields qui puissent avoir le focus !!! */
		tmp_fld = FrmGetObjectPtr (form, tmp_index) ;
		switch (event->data.menu.itemID) {
		  case menuItemCut: FldCut (tmp_fld) ; break ;
		  case menuItemPaste: FldPaste (tmp_fld) ; break ;
		  case menuItemCopy: FldCopy (tmp_fld) ; break ;
		}
	    }
	    return TRUE;
	}
      default:
	FrmHandleEvent(FrmGetActiveForm(), event);
        return TRUE;
    };
    return FALSE;
}

Boolean MainFormEventHandler(EventPtr event)
{ Word currentField;
  FormPtr frm;
  SndPlaySystemSound(sndWarning);
	switch (event->eType)
	{
	  case frmLoadEvent: /// 14/01/2001
	    if (event->data.frmLoad.formID == mainForm) {
	    frm = FrmGetActiveForm();
	    /// Maybe this should not be put at loadTime but at creationTime. No?
	    amountFld = (FieldPtr)GetObjectPtrFromForm(frm, amountField);
	    taxFld = (FieldPtr)GetObjectPtrFromForm(frm, taxRateField);
	    taxAmtFld = (FieldPtr)GetObjectPtrFromForm(frm,taxAmountField);
	    tax2Fld = (FieldPtr)GetObjectPtrFromForm(frm, tax2RateField);
	    tax2AmtFld = (FieldPtr)GetObjectPtrFromForm(frm,tax2AmountField);
	    tipFld = (FieldPtr)GetObjectPtrFromForm(frm, tipRateField);
	    tipAmtFld = (FieldPtr)GetObjectPtrFromForm(frm,tipAmountField);
	    totalFld = (FieldPtr)GetObjectPtrFromForm(frm, totalField);
	    currFld = (FieldPtr)GetObjectPtrFromForm(frm,curr2RateField);
	    currAmtFld = (FieldPtr)GetObjectPtrFromForm(frm,curr2AmountField);
	    //--
	    PutAmountInField(amountFld, 0);
	    if (prefs->taxRate > 0) PutAmountInField(taxFld, prefs->taxRate);
	    else PutAmountInField(taxFld, 0);
	    PutAmountInField(taxAmtFld, 0);
	    if (prefs->tax2Rate > 0) PutAmountInField(tax2Fld, prefs->tax2Rate);
	    else PutAmountInField(tax2Fld, 0);
	    PutAmountInField(tax2AmtFld, 0);
	    if (prefs->tipRate > 0) PutAmountInField(tipFld, prefs->tipRate);
	    else PutAmountInField(tipFld, 0);
	    PutAmountInField(tipAmtFld, 0);
	    PutAmountInField(totalFld, 0);
	    PutAmountInField(currFld, 100);
	    PutAmountInField(currAmtFld, 0);
	    }
            return TRUE;

	  case frmOpenEvent:
	    frm = FrmGetActiveForm();
	    FrmDrawForm(frm);
	    if (event->data.frmOpen.formID == mainForm) 
	    //{
		// Set the handlers for all the fields...
		// FldSetTextHandle(amountFld,(Handle)XX);
		// or Get the penUpEvent (as good)
		SetFocus( ( GetAmountFromField(tipFld) > 0) ? amountField : tipRateField );
	    //};
            return TRUE;

	  case menuEvent:
	    /// Reinstated 14/01/2001
	    MenuEventHandler(event);
            return TRUE;

	  case fldEnterEvent:
	    currentField = event->data.fldEnter.fieldID;
	    //-- Maybe add the current treatment?
            return TRUE;
	    
	  case penUpEvent:
	    //-- Focus changed from a field by the pop
	    //if ( gbMainTestPopTriggerSelected ) {
	//	FldDrawField(gbMainTestPopTriggerSelected); 
	//	FrmSetFocus( FrmGetActiveForm(), currentField);
	//	gbMainTestPopTriggerSelected = NULL;
	    //};
	    //-- Recompute the values
	    // Basically all the same since we do not care about speed, but place!!
	    switch ( currentField ) {
	      case amountField: //alpha
		computeValues();
		break;
	      case taxRateField: //a
		computeValues();
		break;
	      case taxAmountField: //beta
		tax = GetAmountFromField(taxFld);
		COMPA();
		computeValues();
		break;
	      case tax2RateField: //b
		computeValues();
		break;
	      case tax2AmountField: //gamma
		tax2 = GetAmountFromField(tax2Fld);
		COMPB();
		computeValues();
		break;
	      case tipRateField: //c
		computeValues();
		break;
	      case tipAmountField: //delta
		tip = GetAmountFromField(tipFld);
		COMPC();
		computeValues();
		break;
	      case totalField: //epsilon
		total = GetAmountFromField(totalFld);
		// THIS ONE IS A SPECIAL ONE
		RETROCOMPALPHA()
		computeValues();
		break;
	      case curr2AmountField: //rau
		ctotal = GetAmountFromField(totalFld);
		// SAME FOR THIS ONE
		RETROCOMPEPSILON();
		RETROCOMPALPHA()
		computeValues();
		break;
	      case curr2RateField: //d
		computeValues();
		break;
	    }
	    gbMainTestPopTriggerSelected = NULL;
            return TRUE; //? I must leave the opportunity for the system to set the focus(not efficient)
	    
///Suppressed comments
	  // We must do with the Triggers
	  case popSelectEvent:
	    currentField = event->data.popSelect.listID;
	    switch ( currentField ) {
		// When it is the trigger for T/T/C, reset the associated list
	      // Must consider all the triggers (do a function for that)
	      case taxList:
		gbMainTestPopTriggerSelected = taxFld;
		// Must be replaced by database lookup
		switch (event->data.popSelect.selection) {
		    case 1: PutAmountInField(taxFld, 19.6);break ;// prefs->taxRate=19.6;
		    case 2: PutAmountInField(taxFld, 450);break ;// prefs->taxRate=4.5;
		    default: PutAmountInField(taxFld, 0);// prefs->taxRate=0; 
		}
		// PutAmountInField(taxFld, prefs->taxRate);
		computeValues();
                break ;
	    };
            // This refreshes the display correctly
	    FrmHandleEvent(FrmGetActiveForm(), event);
            return TRUE;
///End of suppressed comments	    
	  case ctlSelectEvent:
	    switch (event->data.ctlSelect.controlID)
	    {
	      case calcBtn:
		computeValues();
                return TRUE;

	      case recBtn:
		prefs->taxRate = GetAmountFromField(taxFld);
		prefs->tax2Rate = GetAmountFromField(tax2Fld);
		prefs->tipRate = GetAmountFromField(tipFld);
                return TRUE;

	      case resBtn:
		PutAmountInField(taxFld, prefs->taxRate);
		PutAmountInField(tax2Fld, prefs->tax2Rate);
		PutAmountInField(tipFld, prefs->tipRate);
		SetFocus(amountField);
		computeValues();
                return TRUE;

	      case clearBtn:
		PutAmountInField(amountFld, 0);
		PutAmountInField(taxFld, 0);
		PutAmountInField(taxAmtFld, 0);
		PutAmountInField(tax2Fld, 0);
		PutAmountInField(tax2AmtFld, 0);
		PutAmountInField(tipFld, 0);
		PutAmountInField(tipAmtFld, 0);
		PutAmountInField(totalFld, 0);
		PutAmountInField(currFld, 100);
		PutAmountInField(currAmtFld, 0);
		// Must also reset the chooser (to none or choosed value)
		// THESE COMMENTED RESETS CAUSED A FATAL ERROR IN THE IMPLEMENTED PROGRAMM...(noninit?)
		//for (error = 0; error < 6; error++)
		//    SetTextById(amountField + error, "");
		// These may be handlers? FltSetTextHandle( taxFld, (Handle)prefs->taxRate )
		// sauf que taxRate est un entier et non une chaine
		//if (prefs->tipRate >= 0) PutAmountInField(tipFld, prefs->tipRate);
		//if (prefs->taxRate >= 0) PutAmountInField(taxFld, prefs->taxRate);
		//if (prefs->tax2Rate >= 0) PutAmountInField(tax2Fld, prefs->tax2Rate);
		SetFocus(amountField);
		FrmDrawForm(FrmGetActiveForm());
                return TRUE;

	      case okAboutBtn:
		FrmGotoForm(mainForm);
                return TRUE;
	
	      case delSelectButton:
		// Del the selected item
                return TRUE;
		
	      case newSelectButton:
		// Just brig the new form
                FrmGotoForm( 5100 + LstGetSelection((ListPtr)GetObjectPtrFromForm(FrmGetActiveForm(), ttceCategoryList)) );//taxEditForm==5100
                return TRUE;
		
	      case modSelectButton:
		// Set a variable on the selected stuff and do no do anything if it does not exists.
		FrmGotoForm( 5100 + LstGetSelection((ListPtr)GetObjectPtrFromForm(FrmGetActiveForm(), ttceCategoryList)) );//taxEditForm==5100
                return TRUE;
		
	      case okSelectButton:
		FrmGotoForm(mainForm);
                return TRUE;
	    }

	  default:
	    FrmHandleEvent(FrmGetActiveForm(), event);
            return TRUE;
	}
	return FALSE;
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    EventType evt;
    Word error, prefsSize;
    FormPtr frm;
    Long intVal;
    Int type;

    if (cmd != sysAppLaunchCmdNormalLaunch) return FALSE;

    type = PrefGetPreference(prefNumberFormat);
    if (type==nfPeriodComma || type==nfSpaceComma || type==nfApostropheComma)
    {
	deciChr = deciSep[0] = deciNoDime[0] = ',';
    }

    FrmGotoForm(mainForm);

    prefsSize = sizeof(TipPrefsType);
    prefs = MemPtrNew(sizeof(TipPrefsType));
    if (PrefGetAppPreferences(CREATOR, PREF_VERSION,
			      prefs, &prefsSize, true) == noPreferenceFound)
    {
	prefs->tipRate = prefs->taxRate = prefs->tax2Rate = prefs->currRate = 0;
	prefs->useTax = false;
    }

    do {
	EvtGetEvent(&evt, evtWaitForever);
	if (SysHandleEvent(&evt)) continue;
	if (MenuHandleEvent(NULL, &evt, &error)) continue;
	
        if (evt.eType == frmLoadEvent)
        {
                Word Id = evt.data.frmLoad.formID;
		frm = FrmInitForm(Id) ;
		FrmSetActiveForm(frm) ;
		switch (Id) {
		  case mainForm: // can be put in default
		  case aboutForm:
		  case editListForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)MainFormEventHandler) ;
		    break;
		  case taxEditForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)TaxFormEventHandler) ;
		    break;
		  case tipEditForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)TaxFormEventHandler) ;
		    break;
		  case currEditForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)TaxFormEventHandler) ;
		    break;
		}
        } //else FrmHandleEvent(FrmGetActiveForm(), &evt);
	FrmDispatchEvent(&evt);
    } while ( evt.eType != appStopEvent );
    PrefSetAppPreferences(CREATOR, PREF_VERSION, PREF_VERSION,
				      prefs, sizeof(TipPrefsType), true);
    MemPtrFree(prefs);
    // Unset the handlers for all the fields...
    FldSetTextHandle(amountFld,NULL);
    FldSetTextHandle(tipFld,NULL);
    FldSetTextHandle(taxFld,NULL);
    FldSetTextHandle(tax2Fld,NULL);
    FldSetTextHandle(totalFld,NULL);
    FldSetTextHandle(currFld,NULL);
    FrmCloseAllForms();
    return FALSE;
}
